#!/usr/bin/python3

import json
import os
import requests
import sys

from . import hetzner_dns_helpers as helpers

from .zone_list import zone_list


def zone_get(hetzner_dns_token=None,
             zone_id=None,
             name=None,
             zone_name=None,
             id_only=False):
    """
    Get info about an existing zone.
    https://dns.hetzner.com/api-docs/#operation/GetZone

    Required Parameters: One of: `zone_id` or `name/zone_name`
      - `name` and `zone_name` are interchangeable in all zone functions
    Optional Parameters: `id_only`


    - Lookups can be performed using 'zone_id' or 'name/zone_name'.

    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    - If 'zone_id' passed in args or as environment variable (ZONE_ID),
      then use it to acquire the desired zone.

    - If (domain) 'name/zone_name' passed in args or as environment
      variable (NAME or ZONE_NAME), then use it to acquire the desired
      zone.

    - If 'id_only' passed in args or as environment variable (ID_ONLY),
      return just the zone ID if one exists.

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if os.environ.get('SHOW_HELP'):
        # print the docstring and exit
        print(zone_get.__doc__)
        sys.exit(0)

    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if (name is None and os.environ.get('NAME'))\
            or (zone_name is None and os.environ.get('ZONE_NAME')):
        # get name/zone_name from environment variable
        zone_name = os.environ['ZONE_NAME']\
            if os.environ.get('ZONE_NAME') else os.environ.get('NAME')

    if not id_only and os.environ.get('ID_ONLY'):
        # get id_only from environment variable
        id_only = os.environ['ID_ONLY']

    zone = None

    # if zone_name exists, use it to obtain the zone (skip if zone_id exists)
    if (zone_name or 'ZONE_NAME' in os.environ) and zone_id is None:

        # get list of zones
        response_dict = zone_list()

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # check for matching zone
        dns_zones = response_dict['zones']
        for dns_zone in dns_zones:
            if dns_zone['name'] == zone_name:
                zone = {'zone': dns_zone}
                zone_id = zone['zone']['id']
                break

        # if no matching zone found, halt and notify of error
        if zone is None:
            error_message = "zone not found"

            if __name__ == '__main__':
                print(f"Error: {error_message}")
                sys.exit(1)  # exit with error
            else:
                raise ValueError(error_message)

    # if we don't have a zone_id by now, then get it from environment variable
    if zone_id is None:
        zone_id = os.environ.get('ZONE_ID')

    # raise an exception if no zone_id or zone_name have been passed
    if not zone_id and not zone_name:
        helpers.exit_with_error("Must specify one of: zone_id, zone_name")

    # get response
    try:
        response = requests.get(
            url=f'https://dns.hetzner.com/api/v1/zones/{zone_id}',
            headers={'Auth-API-Token': hetzner_dns_token,
                     'Content-Type': 'application/json; charset=utf-8'})

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # assign value of response_dict to zone
        zone = response_dict

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)

    # return the expected zone ID or zone
    if id_only:
        # return the zone_id
        zone_id = zone['zone']['id']

        if __name__ == '__main__':
            print(zone_id)
            sys.exit(0)  # exit successfully

        return zone_id
    else:
        # return all zone data
        if __name__ == '__main__':
            print(json.dumps(zone))
            sys.exit(0)  # exit successfully

        return zone


if __name__ == '__main__':
    zone_get()



#!/usr/bin/python3

import json
import os
import sys
import requests

from . import hetzner_dns_helpers as helpers


def zone_list(hetzner_dns_token=None):
    """
    Get list of all zones.
    https://dns.hetzner.com/api-docs/#operation/GetZones


    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if os.environ.get('SHOW_HELP'):
        # print the docstring and exit
        print(zone_list.__doc__)
        sys.exit(0)

    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    # get response
    try:
        response = requests.get(url='https://dns.hetzner.com/api/v1/zones',
                                headers={'Auth-API-Token': hetzner_dns_token})

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        if __name__ == '__main__':
            # when running via the terminal, print output to console
            print(decoded_response)
            sys.exit(0)  # exit successfully

        return response_dict

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


if __name__ == '__main__':
    zone_list()



#!/usr/bin/python3

import json
import os
import sys
import requests

from . import hetzner_dns_helpers as helpers
from .zone_list import zone_list


def record_create(hetzner_dns_token=None,
                  record_type=None,
                  name=None,
                  value=None,
                  ttl=None,
                  zone_id=None,
                  zone_name=None,
                  id_only=False):
    """
    Create a new record.
    https://dns.hetzner.com/api-docs/#operation/CreateRecord

    Required Parameters:
      - `hetzner_dns_token`, `name`, `record_type`, `value`, `zone_id`

    Optional Parameters:
      - `zone_name`, `ttl`, `id_only`


    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    * name *MUST* passed in args or as environment variable (NAME).
      It is used to set the name of the new record (e.g. 'www').

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if record_type is None:
        # get record_type from environment variable
        record_type = os.environ['RECORD_TYPE']\
            if os.environ.get('RECORD_TYPE') else os.environ['TYPE']

    if name is None:
        # get name from environment variable
        name = os.environ['NAME']

    if value is None:
        # get value from environment variable
        value = os.environ['VALUE']

    if ttl is None:
        if os.environ.get('TTL'):
            # get ttl from environment variable
            ttl = int(os.environ['TTL'])
        else:
            # use default value for TTL
            ttl = 86400

    if zone_name is None and os.environ.get('ZONE_NAME'):
        # get ttl from environment variable
        zone_name = os.environ['ZONE_NAME']

    # if zone_name exists, use it to obtain the zone_id
    if zone_name:

        # get list of zones
        response_dict = zone_list()

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # check for matching zone
        dns_zones = response_dict['zones']
        for zone in dns_zones:
            if zone['name'] == zone_name:
                zone_id = zone['id']
                break

        # if no matching zone found, then exit with error
        if zone_id is None:
            helpers.exit_with_error("zone not found")

    if not zone_name and not zone_id and not os.environ.get('ZONE_ID'):
        # if neither zone_name or zone_id exist, then exit with error
        helpers.exit_with_error("Must include one of: zone_id, zone_name")
    if zone_id is None:
        # get zone_id from environment variable
        zone_id = os.environ['ZONE_ID']

    try:
        params = {'name': name,
                  'ttl': ttl,
                  'type': record_type,
                  'value': value,
                  'zone_id': zone_id}
        response = requests.post(url='https://dns.hetzner.com/api/v1/records',
                                 headers={'Content-Type': 'application/json',
                                          'Auth-API-Token': hetzner_dns_token},
                                 data=json.dumps(params))

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # return the expected result
        if id_only or os.environ.get('ID_ONLY') == '1':
            # return the zone_id
            result = response_dict['record']['id']

            if __name__ == '__main__':
                print(result)
                sys.exit(0)  # exit successfully

            return result
        else:
            # return all zone data
            if __name__ == '__main__':
                print(json.dumps(response_dict))
                sys.exit(0)  # exit successfully

            return response_dict

        return response_dict

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


if __name__ == '__main__':
    record_create()



#!/usr/bin/python3

import json
import os
import sys
import requests

from . import hetzner_dns_helpers as helpers
from .zone_get import zone_get


def record_list(hetzner_dns_token=None, zone_id=None, zone_name=None):
    """
    Get list of all records.
    https://dns.hetzner.com/api-docs/#operation/GetRecords

    Required Parameters: One of: `zone_id` or `zone_name`


    - Lookups for individual zones can be done using 'zone_name'
      or 'zone_id'.

    - If no 'zone_name' or 'zone_id' is given, all records will
      be returned.

    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    - If using Bash environment variables, ensure that values are
      assigned in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if os.environ.get('SHOW_HELP'):
        # print the docstring and exit
        print(record_list.__doc__)
        sys.exit(0)

    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if zone_id is None and os.environ.get('ZONE_ID'):
        # get zone_id from environment variable
        zone_id = os.environ['ZONE_ID']

    if zone_name is None and os.environ.get('ZONE_NAME'):
        # get zone_name from environment variable
        zone_name = os.environ['ZONE_NAME']

    # if zone_name exists, use it to obtain zone (skip if zone_id exists)
    if (zone_name or 'ZONE_NAME' in os.environ) and not zone_id:

        try:
            # get the desired zone
            response_dict = zone_get(zone_name=zone_name)
        except ValueError:
            # if no matching zone found, halt and notify of error
            helpers.exit_with_error("record not found")

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # get the zone_id
        zone = response_dict['zone']
        zone_id = zone['id']

    # get zone_id from environment variable
    if zone_id is None:
        zone_id = os.environ.get('ZONE_ID')

    try:
        # build params dict
        params = {}
        if zone_id:
            params['zone_id'] = zone_id

        # get response
        response = requests.get(url='https://dns.hetzner.com/api/v1/records',
                                params=params,
                                headers={'Auth-API-Token': hetzner_dns_token})

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        if __name__ == '__main__':
            # when running via the terminal, print output to console then exit
            print(decoded_response)
            sys.exit(0)  # exit successfully

        return response_dict

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


if __name__ == '__main__':
    record_list()



#!/usr/bin/python3

import json
import os
import sys
import requests

from . import hetzner_dns_helpers as helpers


def zone_create(hetzner_dns_token=None,
                name=None,
                id_only=False,
                ttl=None,
                zone_name=None):
    """
    Create a new zone.
    https://dns.hetzner.com/api-docs/#operation/CreateZone

    Required Parameters: `name/zone_name`
      - `name` and `zone_name` are interchangeable in all zone functions
    Optional Parameters: `ttl`


    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    * name *MUST* passed in args or as environment variable.
      It is used to set the (domain) name of the new zone.

      - TIP: zone_create and zone_delete allow ZONE_NAME and NAME to
        be used interchangeably.

    - If 'id_only' passed in args or as environment variable (ID_ONLY),
      return just the zone ID after creating the new zone.

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if os.environ.get('SHOW_HELP'):
        # print the docstring and exit
        print(zone_create.__doc__)
        sys.exit(0)

    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if name is None:
        # get domain name from environment variable
        name = os.environ['ZONE_NAME']\
            if os.environ.get('ZONE_NAME') else os.environ['NAME']
    elif zone_name:
        # allow zone_name and name to be used interchangeably
        # for zone_create and zone_delete
        name = zone_name

    if ttl is None:
        if os.environ.get('TTL'):
            # get TTL from environment variable
            ttl = int(os.environ['TTL'])
        else:
            # use default value for TTL
            ttl = 86400

    try:
        response = requests.post(url='https://dns.hetzner.com/api/v1/zones',
                                 headers={'Content-Type': 'application/json',
                                          'Auth-API-Token': hetzner_dns_token},
                                 data=json.dumps({'name': name,
                                                  'ttl': ttl}))

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # return the expected value
        if id_only or os.environ.get('ID_ONLY') == '1':
            # return the zone_id
            result = response_dict['zone']['id']

            if __name__ == '__main__':
                print(result)
                sys.exit(0)  # exit successfully

            return result
        else:
            # return all zone data
            if __name__ == '__main__':
                print(json.dumps(response_dict))
                sys.exit(0)  # exit successfully

            return response_dict

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


if __name__ == '__main__':
    zone_create()
